/*************************************************************************
*  Copyright (C) 2004 by Janek Kozicki                                   *
*  cosurgi@berlios.de                                                    *
*                                                                        *
*  This program is free software; it is licensed under the terms of the  *
*  GNU General Public License v2 or later. See file LICENSE for details. *
*************************************************************************/

#include "InteractingHollowCylinder2InteractingSphere4SpheresContactGeometry.hpp"
#include "InteractingHollowCylinder.hpp"
#include "SpheresContactGeometry.hpp"
#include "SDECLinkGeometry.hpp"
#include <yade/yade-package-common/InteractingSphere.hpp>

bool InteractingHollowCylinder2InteractingSphere4SpheresContactGeometry::go(	const shared_ptr<InteractingGeometry>& cm1,
										const shared_ptr<InteractingGeometry>& cm2,
										const Se3r& se31,
										const Se3r& se32,
										const shared_ptr<Interaction>& c)
{
///////////////////////////////////////////////////////
//
// so you have to write this function, so that it will correctly calculate SpheresContactGeometry between a sphere
// and a hollow cylinder. Currently here is just slightly omdified version (so it is compiling) of calculating
// SpheresContactGeometry between two spheres. This is obviously wrong :)
//
// you need need to figure out the correct formulas for all components of SpheresContactGeometry:
//
//		Vector3r	 normal			// normal vector of the contact plane between two bodies
//				,contactPoint;		// contact point between two bodies
//
//		Real		 radius1		// radius of first sphere
//				,radius2		// radius of second sphere
//				,penetrationDepth;	// depth of penetration
//
// It means that you can check whether sphere and a cylinder are really touching each other, if not - then you can
// just    return false;  from this function. It will happen if the sphere is in the middle of a cylinder without touching its
// walls. or when a sphere is outside of a cylinder but within the reach of its AABB (Axis Aligned Boundig Box)
//
// if they are in touch, then you have to calculate the normal of the contact plane, the coordinates of the contact point.
// and also depth of penetration.
//
// more interesting is calculation of radius1 and radius2.
//
// for example when a collision between a box and a sphere occurs then (according to suggestion from prof. Donze) it is treated
// as a colision between a sphere and a "mirror sphere" with double radius of the first sphere. You can check that
// in file InteractingBox2InteractingSphere4SpheresContactGeometry. So in fact radius1 and radius2 is a kind of 
// 'characteristic length' not the real radius.
//
// so for a collision of a sphere and a hollow cyllinder let's assume the following:
// - if the collision is in such a way that sphere is outside of a cylinder, then raduis1 is outsideRadius of the cylinder
// - if the collision is in such a way that sphere is INSIDE  of a cylinder, then raduis1 is 5*outsideRadius of the cylinder (it's arbitrary in fact)
// - if the collision is in such a way that sphere is touching the edge of a cylinder, then raduis1 is 2*radius of a sphere
//
// radius2 is always the radius of a sphere.
//
///////////////////////////////////////////////////////////

// obtain access to InteractingHollowCylinder and InteractingSphere, that are in interaction:
	InteractingHollowCylinder* hc = static_cast<InteractingHollowCylinder*>(cm1.get());
	InteractingSphere* s2 = static_cast<InteractingSphere*>(cm2.get());

// normal of the contact plane (currently totally wrong, because it is a formula used between two spheres)
	Vector3r normal = se32.position-se31.position;
// this penetration depth is also totally wrong :)
	Real penetrationDepth = hc->insideRadius+s2->radius-normal.normalize();

	shared_ptr<SpheresContactGeometry> scm;
	if (c->interactionGeometry) // if class Interaction already has interactionGeometry
		scm = dynamic_pointer_cast<SpheresContactGeometry>(c->interactionGeometry); // then use it
	else
		scm = shared_ptr<SpheresContactGeometry>(new SpheresContactGeometry()); // otherwise create new one
		
	if (penetrationDepth>0)
	{
// coordinates of contact point of course are wrong :)
		scm->contactPoint = se31.position+(hc->outsideRadius-0.5*penetrationDepth)*normal;
		scm->normal = normal;
		scm->penetrationDepth = penetrationDepth;
		scm->radius1 = hc->insideRadius;
		scm->radius2 = s2->radius;
				
		if (!c->interactionGeometry)
			c->interactionGeometry = scm;
	
		return true;
	}
	else
		return false;
}


bool InteractingHollowCylinder2InteractingSphere4SpheresContactGeometry::goReverse(	const shared_ptr<InteractingGeometry>& cm1,
											const shared_ptr<InteractingGeometry>& cm2,
											const Se3r& se31,
											const Se3r& se32,
											const shared_ptr<Interaction>& c)
{
	bool isInteracting = go(cm2,cm1,se32,se31,c);
	if (isInteracting)
	{
		SpheresContactGeometry* scm = static_cast<SpheresContactGeometry*>(c->interactionGeometry.get());
		scm->normal = -scm->normal;  // negative normal, because it's REVERSE
		Real tmpR  = scm->radius1;
		scm->radius1 = scm->radius2; // swap raduis1 with radius2, because it's REVERSE contact
		scm->radius2 = tmpR;
	}
	return isInteracting;
}

