/*************************************************************************
*  Copyright (C) 2004 by Janek Kozicki                                   *
*  cosurgi@berlios.de                                                    *
*                                                                        *
*  This program is free software; it is licensed under the terms of the  *
*  GNU General Public License v2 or later. See file LICENSE for details. *
*************************************************************************/

#ifndef CYLINDERTEST_HPP
#define CYLINDERTEST_HPP

#include <yade/yade-core/FileGenerator.hpp>

class CylinderTest : public FileGenerator
{
	private :
		Vector3r	 nbSpheres
				,groundSize
				,gravity;

		bool		 useSpheresAsGround
				,rotationBlocked;

		Real		 minRadius
				,density
				,maxRadius
				,dampingForce
				,disorder
				,dampingMomentum
				,spheresHeight
				,sphereYoungModulus
				,spherePoissonRatio
				,sphereFrictionDeg
				,insideRadius
				,outsideRadius
				,height;

		int		 timeStepUpdateInterval;
	
		void createBox(shared_ptr<Body>& body, Vector3r position, Vector3r extents);
		void createSphere(shared_ptr<Body>& body, int i, int j, int k);
		void createGroundSphere(shared_ptr<Body>& body,Real radius, Real i, Real j, Real k);
		void createCylinder(shared_ptr<Body>& body, Real insideRadius, Real outsideRadius, Real height);
		void createActors(shared_ptr<MetaBody>& rootBody);
		void positionRootBody(shared_ptr<MetaBody>& rootBody);

	// construction
	public :
		CylinderTest ();
		~CylinderTest ();
		string generate();

	protected :
		virtual void postProcessAttributes(bool deserializing);
		void registerAttributes();
	REGISTER_CLASS_NAME(CylinderTest);
	REGISTER_BASE_CLASS_NAME(FileGenerator);
};

REGISTER_SERIALIZABLE(CylinderTest,false);

#endif // CYLINDERTEST_HPP


