/*************************************************************************
*  Copyright (C) 2004 by Janek Kozicki                                   *
*  cosurgi@berlios.de                                                    *
*                                                                        *
*  This program is free software; it is licensed under the terms of the  *
*  GNU General Public License v2 or later. See file LICENSE for details. *
*************************************************************************/

#include "GLDrawInteractingHollowCylinder.hpp"
#include "InteractingHollowCylinder.hpp"
#include <yade/yade-lib-opengl/OpenGLWrapper.hpp>

void GLDrawInteractingHollowCylinder::go(const shared_ptr<InteractingGeometry>& gm, const shared_ptr<PhysicalParameters>&)
{
	// FIXME : check that : one of those 2 lines are useless
  	glMaterialv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, gm->diffuseColor);
	glColor3v(gm->diffuseColor);
	
	InteractingHollowCylinder* hc = (static_cast<InteractingHollowCylinder*>(gm.get()));

	const float nbSteps = 15.0; // FIXME - that should be a parameter from RenderingEngine ....
	Real outsideRadius = hc->outsideRadius;
	Real insideRadius = hc->insideRadius;
	Real height2 = hc->height*0.5;
	
//	if (gm->wire || wire)
//	{
//		glDisable(GL_LIGHTING);
//		glutWireCube(1);
//	}
//	else
	{
		glBegin(GL_QUAD_STRIP);
		glEnable(GL_LIGHTING);

		float pi2 = M_PI*2.0;
		float increment = pi2 / nbSteps;
		float s1,c1,s2,c2,s1o,c1o,s2o,c2o;
		for(float ang=0 ; ang <= pi2 ; ang += increment )
		{
			if(ang==0)
			{
				s1o = sin(ang)*outsideRadius;
				c1o = cos(ang)*outsideRadius;
				s2o = sin(ang)*insideRadius;
				c2o = cos(ang)*insideRadius;
			} else
			{
				s1 = sin(ang)*outsideRadius;
				c1 = cos(ang)*outsideRadius;
				s2 = sin(ang)*insideRadius;
				c2 = cos(ang)*insideRadius;
			
				glNormal3f( (s1+s1o)*0.5, (c1+c1o)*0.5, 0 );
				glVertex3f(s1o, c1o, height2 );
				glVertex3f(s1 , c1 , height2 );
				glNormal3f(0,0,-1);
				glVertex3f(s1o, c1o,-height2 );
				glVertex3f(s1 , c1 ,-height2 );
	
				glNormal3f(-(s1+s1o)*0.5,-(c1+c1o)*0.5, 0 );
				glVertex3f(s2o, c2o,-height2 );
				glVertex3f(s2 , c2 ,-height2 );
				glNormal3f(0,0, 1);
				glVertex3f(s2o, c2o, height2 );
				glVertex3f(s2 , c2 , height2 );
				glVertex3f(s1o, c1o, height2 );
				glVertex3f(s1 , c1 , height2 );
	
				s1o = s1;
				c1o = c1;
				s2o = s2;
				c2o = c2;
			}
		}
	
		glEnd();
	}
}

